<?php

/*
slug: backupRepository
version: 1.2.3
*/

class addonBackupRepository{
	
	private static $version = '1.2.3';

	public static function version(){
		return self::version;	
	}
	
	public static function init(){
		require_once(APP_ROOT."/addons/backupRepository/controllers/func.php");	
		require_once(APP_ROOT."/addons/backupRepository/controllers/manageClientsBackupRepository.php");
		panelRequestManager::addFunctions('backupRepositoryTestConnection', 'backupRepositorySaveProfile', 'backupRepositoryGetProfile', 'backupRepositoryGetAllProfiles', 'backupRepositoryDeleteProfile', 'backupRepositoryGetDropBoxKeys', 'backupRepositoryUpdateDropBoxKeys', 'backupRepositorySaveGoogleDriveCreds', 'backupRepositoryGetGoogleDriveCreds','backupRepositoryGetDropBoxAccess', 'backupRepositorySaveDropBoxAccountDetails', 'backupRepositoryUpgradeSchedules','backupRepositoryDropbox');
		regHooks('responseProcessors', 'taskTitleTemplate', 'headHTMLFiles');
	}	
	
	public static function install(){
		
		$Q1 = DB::doQuery("CREATE TABLE IF NOT EXISTS `?:backup_repository` (
  `repositoryID` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `profileName` varchar(50) NOT NULL,
  `type` varchar(255) NOT NULL,
  `credentials` text NOT NULL,
  `status` varchar(50) DEFAULT NULL,
  `errorNo` int(10) unsigned DEFAULT NULL,
  `errorMsg` varchar(255) DEFAULT NULL,
  PRIMARY KEY (`repositoryID`),
  UNIQUE KEY `profileName` (`profileName`)
)".DB::getSQLTableEnv().";");

		if($Q1){
			return true;	
		}
		return false;
	}

	public static function update(){
		if (version_compare(getOldAddonVersion('backupRepository'), '1.1.0beta2', '<')){
			$oldToNewEndpoints = array(
				's3.amazonaws.com'							=> '',
				's3-us-west-2.amazonaws.com'				=> 'us-west-2',
				's3-us-west-1.amazonaws.com'				=> 'us-west-1',
				's3-eu-west-1.amazonaws.com'				=> 'eu-west-1',
				's3-ap-southeast-1.amazonaws.com'			=> 'ap-southeast-1',
				's3-website-ap-southeast-2.amazonaws.com'	=> 'ap-southeast-2',
				's3-ap-northeast-1.amazonaws.com'			=> 'ap-northeast-1',
				's3-website-sa-east-1.amazonaws.com'		=> 'sa-east-1',
				);
			self::updateEndpointsAndSiteFolderInScheduleBackups($oldToNewEndpoints);
			self::updateEndpointsAndSiteFolderInProfiles($oldToNewEndpoints);
		}
		if(version_compare(getOldAddonVersion('backupRepository'), '1.1.2beta1', '<')){
			$deleteFiles = array('/addons/backupRepository/initHTMLHead.php' => array('type' => 'f') );
			fileSystemRemoveFiles($deleteFiles);
		}
	}

	public static function updateEndpointsAndSiteFolderInScheduleBackups($oldToNewEndpoints){
		if (!DB::getExists("SHOW TABLES LIKE '?:backup_schedules'")) {
			return false;
		}
		$scheduledBackups = DB::getArray("?:backup_schedules", "scheduleID, additionalDatas", "1");
		if (empty($scheduledBackups)) {
			return '';
		}
		foreach ($scheduledBackups as $key => $scheduledBackup) {
			$scheduleID = $scheduledBackup['scheduleID'];
			$scheduledBackupData = unserialize($scheduledBackup['additionalDatas']);
			if (isset($scheduledBackupData['secure']['account_info']['iwp_amazon_s3'])) {
				$bucketRegion = $scheduledBackupData['secure']['account_info']['iwp_amazon_s3']['as3_bucket_region'];
				if (!array_key_exists('as3_site_folder', $scheduledBackupData['secure']['account_info']['iwp_amazon_s3'])) {
					$scheduledBackupData['secure']['account_info']['iwp_amazon_s3']['as3_site_folder'] = '';
				}
				 if(array_key_exists($bucketRegion, $oldToNewEndpoints)){
					$scheduledBackupData['secure']['account_info']['iwp_amazon_s3']['as3_bucket_region'] = $oldToNewEndpoints[$bucketRegion] ;
				}
			} else if(isset($scheduledBackupData['secure']['account_info']['iwp_ftp'])){
				if (!array_key_exists('ftp_site_folder', $scheduledBackupData['secure']['account_info']['iwp_ftp'])) {
					$scheduledBackupData['secure']['account_info']['iwp_ftp']['ftp_site_folder'] = '';
				}
			} else if(isset($scheduledBackupData['secure']['account_info']['iwp_dropbox'])){
				if (!array_key_exists('dropbox_site_folder', $scheduledBackupData['secure']['account_info']['iwp_dropbox'])) {
					$scheduledBackupData['secure']['account_info']['iwp_dropbox']['dropbox_site_folder'] = '';
				}
			}
			$serializedData = serialize($scheduledBackupData);
			$where = array(
				'query' =>   "scheduleID=':scheduleID'",
				'params' => array(
					':scheduleID'=>$scheduleID
				)
			);
			$isDone = DB::update("?:backup_schedules", array('additionalDatas' => $serializedData), $where);
			if (!$isDone) {
				die('<br>SQL Error: update failed.');
			}
		}
	}

	public static function updateEndpointsAndSiteFolderInProfiles($oldToNewEndpoints){
		$backupProfiles = DB::getArray("?:backup_repository", "repositoryID, credentials, type", "1");
		if (empty($backupProfiles)) {
			return '';
		}
		foreach ($backupProfiles as $key => $backupProfile) {
			$repositoryID = $backupProfile['repositoryID'];
			$backupProfileData = unserialize($backupProfile['credentials']);
			$backupProfileType = $backupProfile['type'];
			if ($backupProfileType === 'iwp_amazon_s3') {
				$bucketRegion = $backupProfileData['as3_bucket_region'];
				if (!array_key_exists('as3_site_folder',$backupProfileData)) {
					$backupProfileData['as3_site_folder'] = '';
				}
				 if(array_key_exists($bucketRegion, $oldToNewEndpoints)){
					$backupProfileData['as3_bucket_region'] = $oldToNewEndpoints[$bucketRegion] ;
				} 
			} else if($backupProfileType === 'iwp_ftp'){
				if (!array_key_exists('ftp_site_folder',$backupProfileData)) {
					$backupProfileData['ftp_site_folder'] = '';
				}
			} else if($backupProfileType === 'iwp_dropbox'){
				if (!array_key_exists('dropbox_site_folder',$backupProfileData)) {
					$backupProfileData['dropbox_site_folder'] = '';
				}
			}
			
			$serializedData = serialize($backupProfileData);
			$where = array(
				'query' =>   "repositoryID=':repositoryID'",
				'params' => array(
					':repositoryID'=>$repositoryID
				)
			);
			$isDone = DB::update('?:backup_repository', array('credentials'=>$serializedData), $where);
			if (!$isDone) {
				die('<br>SQL Error: update failed.');
			}
		}
	}
}
?>