<?php
class manageClientsClientReporting {
	
	public static function runReportProcessor($siteIDs = array(), $params){
		$paramsResult = fetchReportDetailUsingID($params);
		$paramsParsed = unserialize(base64_decode($paramsResult['params']));		
		returnScheduledReportDates($paramsParsed);		
		$paramsParsed['headerFooterLogoContNew']['headerContent'] 		= htmlentities($paramsParsed['headerFooterLogoContNew']['headerContent'],ENT_COMPAT,'UTF-8');
		$paramsParsed['headerFooterLogoContNew']['footerContent'] 		= htmlentities($paramsParsed['headerFooterLogoContNew']['footerContent'],ENT_COMPAT,'UTF-8');

		clientReportingNewHeaderFooterLogoSave($paramsParsed['headerFooterLogoContNew']);			
		
		$paramsRequest = array(
			'action' => $params['action'],
			'actions' => $paramsParsed['actions'],
			'dates' => $paramsParsed['dates'],
			'headerFooterLogoContNew' => $paramsParsed['headerFooterLogoContNew']
		);
		
		if(!empty($params['requiredData'])) {
			$paramsRequest['requiredData'] = $params['requiredData'];
		}
		if(empty($paramsParsed['siteID'])) {
			return false;
		}
		return self::insertReportDetailsToHistory($paramsParsed['siteID'],$paramsRequest,'fetchReportDetails');
	}
	
	public static function runReportResponseProcessor($historyID, $responseData){
		self::responseProcessorForReport($historyID, $responseData);
	}
	
	public static function insertReportDetailsToHistory($siteIDs, $params, $uniqueName) {
		$type = 'clientReporting';
		$action = $params['action'];
		$requestAction = "fetch_activities_log";
		$requiredData = '';
		$isCron = false;
		if(!empty($params['requiredData'])) {
			$requiredData = $params['requiredData'];
			$isCron = true;
		}
		$historyAdditionalData = array();
		$historyAdditionalData[] = array('uniqueName' => $uniqueName, 'detailedAction' => $params['action']);
		$events=1;
		
		if(empty($params['dates'])){
			if($isCron === false) {
				addNotification($type='E', $title='Empty date range', 'Date range you have chosen is empty.', $state='U', $callbackOnClose='', $callbackReference='');
			}
			return false;				
		}
		
		$dates 		= explode('-', $params['dates']);
		$fromDate 	= strtotime(trim($dates[0].' 00:00:00 UTC'));
		$toDate		= strtotime(trim($dates[1].' 23:59:59 UTC'));		
		
		if(empty($fromDate) || empty($toDate) || $fromDate == -1 || $toDate == -1){
			if($isCron === false) {
				addNotification($type='E', $title='Invalid date range', 'Date range you have chosen is not valid.', $state='U', $callbackOnClose='', $callbackReference='');
			}
			return false;				
		}
		
		$actionsRequestArray = $detailed = array();
		$i = 0;
		foreach($params['actions'] as $key => $actionsArray) {
			
			if(isset($actionsArray['mainAction']) && $actionsArray['mainAction']) {
				$detailed[$actionsArray['mainAction']]['count'] = 0;					
			}
			
			if(isset($actionsArray['mainAction']) && $actionsArray['mainAction'] && empty($actionsArray['subActions'])) {
				$actionsRequestArray[] = $actionsArray['mainAction'];
				
			} else if(isset($actionsArray['subActions']) && is_array($actionsArray['subActions']) && count($actionsArray['subActions'])) {
				foreach($actionsArray['subActions'] as $key=>$subActions) {
					if ($subActions == 'translations-updated') {
						continue;
					}
					$actionsRequestArray[] = $subActions;
					
					$detailed[$actionsArray['mainAction']][$subActions]['count'] = 0;
				}
			}
			$i++;
		}			
		
		if(!count($actionsRequestArray)) {
			if($isCron === false) {
				addNotification($type='E', $title='Invalid actions', 'Invalid actions you have chosen.', $state='U', $callbackOnClose='', $callbackReference='');
			}
			return false;					
		}
		
		$sitesCount = 0;
		foreach ($siteIDs as $index => $siteID) {
			$siteData = getSiteData(intval($siteID));
			
			$requestParams = array(
				'detailed' => $detailed,
				'originalActions' => $params['actions'],
				'actions' => $actionsRequestArray,
				'fromDate' => $fromDate,
				'toDate' => $toDate
			);
			
			if(!empty($requiredData)) {
				$requestParams['requiredData'] = $requiredData;
			}			
			
			$PRP = array();
			$PRP['requestAction'] 	= $requestAction;
			$PRP['siteData'] 		= $siteData;
			$PRP['type'] 			= $type;
			$PRP['action'] 			= $action;
			$PRP['requestParams'] 	= $requestParams;
			$PRP['directExecute'] 	= false;
			$PRP['events'] 			= $events;
			$PRP['sendAfterAllLoad'] = true;
			$PRP['historyAdditionalData'] 	= $historyAdditionalData;
			
			$historyID = prepareRequestAndAddHistory($PRP);
			
			if(!empty($requiredData)) {
				updateHistory(array('isPluginResponse' => '0'), $historyID);
			}
			
			if(!empty($requiredData) && $sitesCount == 0) {
				$where = array(
							'query' =>  "historyID = ':historyID'",
							'params' => array(
							   ':historyID' => $historyID
							)
						);				
				$actionID = DB::getField("?:history", "actionID", $where);
				
				DB::insert("?:temp_storage", array('type' => 'scheduledReportTotalSites', 'paramID' => $actionID, 'time' => time(), 'data' =>  count($siteIDs)));
				
				DB::insert("?:temp_storage", array('type' => 'scheduledReportCronSitesTotalSucceeded', 'paramID' => $actionID, 'time' => time(), 'data' =>  '0'));				
			}
			
			$sitesCount++;
		}			
	}
	
	public static function responseProcessorForReport($historyID, $responseData) {
		$whereHistoryID = array(
					'query' =>  "historyID = ':historyID'",
					'params' => array(
					   ':historyID' => $historyID
					)
				);		
		$savedRequest = DB::getField("?:history_raw_details", "request", $whereHistoryID);
		$savedRequest = unserialize(base64_decode($savedRequest));
		
		$whereHistoryData = array(
					'query' =>  "historyID = ':historyID'",
					'params' => array(
					   ':historyID' => $historyID
					)
				);
		$historyData = DB::getRow("?:history", "type, action, actionID, siteID", $whereHistoryData);
		
		$actionID = $historyData['actionID'];
		$siteID = $historyData['siteID'];
		$action = $historyData['action'];

		$whereTotalSites = array(
			'query' =>  "type = ':type' AND paramID = ':paramID'",
			'params' => array(
			   ':type' => 'scheduledReportTotalSites',
			   ':paramID' => $actionID
			)
		);			
		$totalSites = DB::getField("?:temp_storage", "data", $whereTotalSites);
		
		$whereTotalSucceeded = array(
			'query' =>  "type = ':type' AND paramID = ':paramID'",
			'params' => array(
			   ':type' => 'scheduledReportCronSitesTotalSucceeded',
			   ':paramID' => $actionID
			)
		);			
		$totalSucceededSites = DB::getField("?:temp_storage", "data", $whereTotalSucceeded);		
		
		if(!empty($savedRequest['params']) && !empty($savedRequest['params']['requiredData']) && function_exists(key($savedRequest['params']['requiredData']))) {
			removeResponseJunk($responseData);
			if (strrpos($responseData, '_IWP_JSON_PREFIX_') !== false) {
				$responseDataArray = explode('_IWP_JSON_PREFIX_', $responseData);
				$responseRawData = $responseDataArray[1];
				$responseData = json_decode(base64_decode($responseRawData), true) ;
			} else{
				$responseData = unserialize(base64_decode($rawResponseData));
			}
		}			
		
		responseDirectErrorHandler($historyID, $responseData);
		
		$response = array();
		if(empty($responseData) || empty($responseData['success']) || !empty($responseData['success'])) { 
			
			if(!empty($responseData['success'])) {
				DB::update("?:history_additional_data", array('status' => 'success'), $whereHistoryID);
			
				$response['_'.$siteID] = $responseData['success'];				
			}			
			
			if(!empty($savedRequest['params']) && !empty($savedRequest['params']['requiredData']) && function_exists(key($savedRequest['params']['requiredData']))) {

				if(($totalSucceededSites + 1) < $totalSites) {
					
					DB::update("?:temp_storage", array('data' => $totalSucceededSites + 1), $whereTotalSucceeded);
					
					DB::insert("?:temp_storage", array('type' => $action, 'paramID' => $actionID, 'time' => time(), 'data' =>  serialize($response)));
					
					return;
				} 
				
				DB::delete("?:temp_storage", $whereTotalSites);
				DB::delete("?:temp_storage", $whereTotalSucceeded);				
			}
	
			DB::insert("?:temp_storage", array('type' => $action, 'paramID' => $actionID, 'time' => time(), 'data' =>  serialize($response)));			

		}
		
		if(!empty($savedRequest['params']) && !empty($savedRequest['params']['requiredData']) && function_exists(key($savedRequest['params']['requiredData']))) {
			$requiredData = array();
			$requiredData = $savedRequest['params']['requiredData'][key($savedRequest['params']['requiredData'])];
			$requiredData['actionID'] = $actionID;
			call_user_func(key($savedRequest['params']['requiredData']), $requiredData);
		}
		return;		
	}
	
	public static function fetchReportDetailsProcessor($siteIDs,$params){
		
		$params['headerFooterLogoContNew']['headerContent'] 		= htmlentities($params['headerFooterLogoContNew']['headerContent'],ENT_COMPAT,'UTF-8');
		$params['headerFooterLogoContNew']['footerContent'] 		= htmlentities($params['headerFooterLogoContNew']['footerContent'],ENT_COMPAT,'UTF-8');

		clientReportingNewHeaderFooterLogoSave($params['headerFooterLogoContNew']);
		
		return self::insertReportDetailsToHistory($siteIDs,$params,'fetchReportDetails');
	}
	public static function fetchReportDetailsResponseProcessor($historyID, $responseData){
		self::responseProcessorForReport($historyID, $responseData);
	}
}
manageClients::addClass('manageClientsClientReporting');