<?php

class scheduleBackupFailedNotification{
	private $userIDs;
	private $failedCount;
	private $totalCount;
	private $affectedRows;
	private $scheduleBackupSummary;
	function __construct() {
		$this->userIDs = array();
		$this->failedCount = 0;
		$this->totalCount = 0;
		$this->affectedRows = 0;
		$this->scheduleBackupSummary = array();
		$this->scheduleBackupFailedNotify();
	}

	function scheduleBackupFailedNotify(){
		$scheduledBackupEmailSetting = scheduledBackupGetEmailSetting();
		$actionID = $this->getActionID();
		if (empty($actionID)) {
			return false;
		}
		$completedTasks = getCompletedTaskIDsByActionID($actionID);
		if (empty($completedTasks)) {
			return false;
		}
		$this->processDataToNotify($completedTasks);

		if (empty($this->scheduleBackupSummary) || empty($this->scheduleBackupSummary['errorSitesDetail'])) {
			return '';
		}
		if (count($this->userIDs) == 0) {
			$this->pushSuperAdminID();
		}
		$this->scheduleBackupSummary['totalCount'] = $this->totalCount;
		$this->scheduleBackupSummary['failedCount'] = $this->failedCount;
		if (!$this->affectedRows) {
			return false;
		}
		$this->appNotification($actionID);
		if ($scheduledBackupEmailSetting != 1) {
			return false;
		}
		$this->sendEmail();
	}
	function processDataToNotify($completedTasks){
		foreach ($completedTasks as $key => $values) {
			$this->totalCount++;
			$historyID 	= $values['historyID'];
			$siteID 	= $values['siteID'];
			$historyAdditionalData = getHistoryAdditionalDataByHistoryID($historyID);
			if($historyAdditionalData[0]['status'] != 'success'){
				$this->parseErrorData($siteID, $historyAdditionalData);
				$this->failedCount++;
			} else {
				$this->parseSuccessData($siteID, $historyAdditionalData);
			}
		}
	}
	function sendEmail(){
		foreach ($this->userIDs as $userID) {
			$params = array(
				'scheduleBackupSummary' => $this->scheduleBackupSummary,
				'userID' => $userID,
			);
			sendAppMail($params, '/addons/scheduleBackup/templates/failedEmailNotify.tpl.php');
		}
	}

	function getActionID(){
		$minDuration = time() - (2 * 60 * 60 * 24);
		$where = array(
			'query'		=> "type = ':scheduleBackup' AND (action = ':runTaskSingleCall' OR action =':runTaskMulticall') AND notifyFailed =':notifyFailed' AND microtimeAdded >= ':timeStamp' GROUP BY actionID ORDER BY historyID ASC",
			'params' 	=> array(
				   ':scheduleBackup' => 'scheduleBackup',
				   ':runTaskSingleCall' => 'runTask',
				   ':runTaskMulticall' => 'multiCallRunTask',
				   ':notifyFailed' => 'pending',
				   ':timeStamp' =>$minDuration,
				)
			);
		$actionIDs = DB::getFields("?:history", "actionID", $where);

		foreach ($actionIDs as $actionID) {
			$taskRunning = isTasksRunningByActionID($actionID);
			if ($taskRunning) {
				continue;
			}
			return $actionID;
		}
	}

	function appNotification($actionID){
		$notificationMessage = '';
		if (empty($this->scheduleBackupSummary)) {
			return false;
		}
		foreach ($this->scheduleBackupSummary['errorSitesDetail'] as $siteName => $siteLogs) {
			if (!empty($notificationMessage)) {
				$notificationMessage .= "<br><br>".$siteName . " - " .$siteLogs['detailedError']['errorMsg'];
			} else {
				$notificationMessage .= $siteName . " - " .$siteLogs['detailedError']['errorMsg'];
			}
		}
		$notificationMessage .= " <a id='scheduleBackupFailedAppNotificationViewActivityLog' actionid = ".$actionID.">View activity log</a>";
		addNotification($type='E', $title='Schedule Backup Failed', $notificationMessage, $state='U', $callbackOnClose='', $callbackReference='');
	}

	function parseErrorData($siteID, $historyAdditionalData){
		$this->parseBackupAndErrorData($siteID, $historyAdditionalData);
		$this->getEmailIDs($siteID);
	}

	function getEmailIDs($siteID){
		//if (!function_exists('multiUserGetAdmin') || !function_exists('getManagersIDsBySiteID')) {
		$this->pushSuperAdminID();
			// return false;
		//}
		if (function_exists('getManagersIDsBySiteID')) {
			$assingedUserIDs = getManagersIDsBySiteID($siteID);
			foreach ($assingedUserIDs as $key1 => $assingedUserID) {
				$this->pushManagersIDs($assingedUserID);
			}
		}
	}
	function pushSuperAdminID(){
		$where = "accessLevel = 'admin' AND status = '1' ORDER BY userID ASC";
		$adminData = DB::getArray("?:users", "*", $where);
		foreach ($adminData as $key => $value) {
			if(!in_array($value['userID'], $this->userIDs)){
				array_push($this->userIDs, $value['userID']);
			}
		}
	}

	function pushManagersIDs($assingedUserID){
		$params = array();
		$params['userID'] = $assingedUserID;
		if(!function_exists('multiUserGetAdmin')){
			return false;
		}
		$userDetails = multiUserGetAdmin($params); // check only multiuser available
		if (empty($userDetails) || !is_array($userDetails)) {
			return false;
		}

		if (empty($userDetails['permissions']['access']) || !is_array($userDetails['permissions']['access']) || $userDetails['status'] == 0) {
			return false;
		}

		if (!in_array("scheduleBackup", $userDetails['permissions']['access']) ) {
			return false;
		}

		if(in_array($userDetails['userID'], $this->userIDs)){
			return false;
		}
		array_push($this->userIDs, $userDetails['userID']);
	}

	function parseBackupAndErrorData($siteID, $historyAdditionalData){
		$scheduduleBackupDetails = $this->addScheduleData($historyAdditionalData);
		$siteData = getSiteData($siteID, 'true');

		$this->scheduleBackupSummary['errorSitesDetail'][$siteData['name']]['detailedError'] = array(
			'error' => $historyAdditionalData[0]['error'],
			'errorMsg' => $historyAdditionalData[0]['errorMsg'],
			);
		$this->changeStatusToSent($historyAdditionalData[0]['historyID']);
	}

	function parseSuccessData($siteID, $historyAdditionalData){
		$scheduduleBackupDetails = $this->addScheduleData($historyAdditionalData);
		$siteData = getSiteData($siteID, 'true');
		$this->scheduleBackupSummary['successSitesDetail'][$siteData['name']]['detailedMesg'] = array(
			'success' => '1',
		);
		$this->changeStatusToChecked($historyAdditionalData[0]['historyID']);
	}

	function changeStatusToChecked($historyID){
		$where = array(
			'query' =>  "historyID = ':historyID' AND notifyFailed = 'pending' ",
			'params' => array(
				':historyID'=>$historyID,
			)
		);
		DB::update("?:history", array("notifyFailed" => 'checked'), $where);
	}

	function changeStatusToSent($historyID){
		$where = array(
			'query' =>  "historyID = ':historyID' AND notifyFailed = 'pending' ",
			'params' => array(
				':historyID'=>$historyID,
			)
		);

		DB::update("?:history", array("notifyFailed" => 'sent'), $where);
		$this->affectedRows = DB::affectedRows();
	}
	function addScheduleData($historyAdditionalData){
		$params = array();
		$params['scheduleKey'] = $historyAdditionalData[0]['uniqueName'];
		$scheduduleBackupDetails = scheduleBackupEdit($params);
		if (empty($this->scheduleBackupSummary['backupName']) && empty($this->scheduleBackupSummary['what'])) {
			$this->scheduleBackupSummary['backupName'] = $scheduduleBackupDetails['scheduleData']['backupName'];
			$this->scheduleBackupSummary['what'] = $this->dataBeautifierForEmail($scheduduleBackupDetails['scheduleData']['additionalDatas']['args']['what'], 'what');
			$this->scheduleBackupSummary['mechanism'] = $this->dataBeautifierForEmail($scheduduleBackupDetails['scheduleData']['additionalDatas']['mechanism'], 'mechanism');
			$this->scheduleBackupSummary['type'] = $scheduduleBackupDetails['scheduleData']['type'];
			$this->scheduleBackupSummary['time'] = $this->dataBeautifierForEmail($scheduduleBackupDetails['scheduleData']['time'], 'time');
			$this->scheduleBackupSummary['day'] = $this->dataBeautifierForEmail($scheduduleBackupDetails['scheduleData']['day'], 'day', $this->scheduleBackupSummary['type']);
			if (empty($scheduduleBackupDetails['scheduleData']['additionalDatas']['secure']['account_info'])) {
				$this->scheduleBackupSummary['repo'] = $this->dataBeautifierForEmail('' , 'repo');
				return $scheduduleBackupDetails;
			}
			foreach ($scheduduleBackupDetails['scheduleData']['additionalDatas']['secure']['account_info'] as $repoName => $repoDetails) {
				$this->scheduleBackupSummary['repo'] = $this->dataBeautifierForEmail($repoName , 'repo');
			}
			return $scheduduleBackupDetails;
		}
	}

	function dataBeautifierForEmail($data, $type, $extra = NULL){
		switch ($type) {
			case 'what':
				if ($data === 'db') {
					return 'DB';
				} else if ($data === 'files') {
					return 'Files';
				} else if ($data === 'full') {
					return 'Files + DB';
				}
				break;
			case 'mechanism':
				if ($data === 'singleCall') {
					return 'Single-call';
				} else if($data === 'multiCall'){
					return 'Multi-call';
				}
				break;
			case 'repo':
				if ($data === 'iwp_ftp') {
					return 'FTP';
				} else if($data === 'iwp_amazon_s3'){
					return 'Amazon S3';
				} else if($data === 'iwp_dropbox'){
					return 'Dropbox';
				} else if($data === 'iwp_gdrive'){
					return 'Google Drive';
				} else {
					return 'WP Server';
				}
				break;
			case 'day':
				if($extra === 'weekly'){
					switch ($data) {
					case 1:
						return "Monday";
					case 2:
						return "Tuesday";
					case 3:
						return "Wednesday";
					case 4:
						return "Thursday";
					case 5:
						return "Friday";
					case 6:
						return "Saturday";
					case 7:
						return "Sunday";
					}
				} else if($extra === 'daily'){
					return '';
				}
				if ($data >= 11 && $data <= 13) {
					return $data."th";
				}
				switch ($data % 10) {
				case 1:
					return $data."st";
				case 2:
					return $data."nd";
				case 3:
					return $data."rd";
				default:
					return $data."th";
				}
			case 'time':
				return	$data>= 12 ? (($data%12 === 0 ) ? 12 : $data%12).' pm' : (($data%12 === 0 ) ? 12 : $data).' am';
		}
	}

}