<?php

class scheduleBackupPreCheck{

	function getAnotherBackupScheduledInSameScheduleKey($scheduleWaitingData, $currentScheduleKey){
		$IDs = array();
		foreach ($scheduleWaitingData as $ID => $scheduleKey) {
			if($currentScheduleKey == $scheduleKey['scheduleKey']){
				$IDs[] = $ID;
			}
		}
		if (!empty($IDs)) {
			return $IDs;
		}
		return false;
	}

	function killOldAndPostPoneCurrentScheduleBackup($anotherBackupHistoryIDs, $currentHistoryID){
		foreach($anotherBackupHistoryIDs as $anotherBackupHistoryID){
			if ($anotherBackupHistoryID < $currentHistoryID) {
				$this->postponeCurrentScheduleBackup($currentHistoryID);
				$this->killScheduleBackup($anotherBackupHistoryID);
			} else {
				$this->postponeCurrentScheduleBackup($anotherBackupHistoryID);
				$this->killScheduleBackup($currentHistoryID);
			}
		}
	}

	function getTimeScheduledbyHistoryID($historyID){
		$where = array(
				'query' =>  "historyID = ':historyID'",
				'params' => array(
					':historyID' => $historyID
				)
		);
		return DB::getField('?:history', 'timeScheduled', $where);
	}

	function postponeCurrentScheduleBackup($historyID){
		$where = array(
				'query' =>  "historyID = ':historyID'",
				'params' => array(
					':historyID' => $historyID
				)
		);
		$currentTimeScheduled = $this->getTimeScheduledbyHistoryID($historyID);
		$postponeTime = 60 * 15; // check after 15 min
		$timeScheduled = $currentTimeScheduled + $postponeTime;
		return DB::update("?:history", array( 'status' => 'scheduled', 'timeScheduled' => $timeScheduled), $where);
	}

	function runningBackupsBySiteID($siteID, $historyID){
		$manualBackupRunningIDs = $this->runningManualBackupsBySiteID($siteID, $historyID);
		$scheduleWaitingIDs = $this->waitingScheduledBackupsBySiteID($siteID, $historyID);
		if (!count($manualBackupRunningIDs)) {
			$scheduleBackupRunningIDs = $this->runningScheduleBackupsBySiteID($siteID, $historyID);
			if (empty($scheduleBackupRunningIDs) && empty($scheduleWaitingIDs)) {
				return 0;
			}
			$finalArray['scheduleRunningCount'] = count($scheduleBackupRunningIDs);
			$finalArray['scheduleWaitingData'] = $this->getWaitingScheduleKeys($scheduleWaitingIDs);
		}
		$finalArray['scheduleWaitingCount'] = count($scheduleWaitingIDs);
		if (is_array($scheduleWaitingIDs)) {
			sort($scheduleWaitingIDs);
			$finalArray['scheduleWaitingPriorityID'] = $scheduleWaitingIDs[0]; 
		}
		if (!count($scheduleWaitingIDs)) {
			$finalArray['manualRunningCount'] = count($manualBackupRunningIDs);
			return $finalArray;
		}
		$finalArray['manualRunningCount'] = count($manualBackupRunningIDs);
		$finalArray['scheduleWaitingData'] = $this->getWaitingScheduleKeys($scheduleWaitingIDs);
		return $finalArray;
	}

	function runningManualBackupsBySiteID($siteID, $historyID){
		$where = array(
			'query'		=> "siteID = :siteID AND historyID != :historyID AND (type = 'backup' AND (action = 'now' OR action = 'multiCallNow'))  AND status NOT IN ('completed', 'netError', 'error') GROUP BY actionID ORDER BY historyID DESC",
			'params' 	=> array(
				   ':siteID' => $siteID,
				   ':historyID' => $historyID,
				)
			);
		return DB::getArray("?:history", "count(historyID)", $where);
	}

	function runningScheduleBackupsBySiteID($siteID, $historyID){
		$where = array(
			'query'		=> "siteID = :siteID AND historyID != :historyID AND (type = 'scheduleBackup' AND (action = 'runTask' OR action ='multiCallRunTask'))  AND status NOT IN ('scheduled', 'completed', 'netError', 'error') GROUP BY actionID ORDER BY historyID DESC",
			'params' 	=> array(
				   ':siteID' => $siteID,
				   ':historyID' => $historyID,
				)
			);
		return DB::getFields("?:history", "historyID", $where);
	}

	function waitingScheduledBackupsBySiteID($siteID, $historyID){
		$where = array(
			'query'		=> "siteID = :siteID AND historyID != :historyID AND (type = 'scheduleBackup' AND (action = 'runTask' OR action ='multiCallRunTask')) AND status IN ('scheduled') GROUP BY actionID ORDER BY historyID DESC",
			'params' 	=> array(
				   ':siteID' => $siteID,
				   ':historyID' => $historyID,
				)
			);
		return DB::getFields("?:history", "historyID", $where);
	}

	function getWaitingScheduleKeys($scheduleBackupRunningIDs){
		$scheduleWaiting = array();
		foreach ($scheduleBackupRunningIDs as $ID) {
			$where = array(
				'query'		=> "historyID = :historyID",
				'params' 	=> array(
					':historyID' => $ID,
				)
			);
			$scheduleKey = DB::getField('?:history_additional_data', 'uniqueName', $where);
			$scheduleWaiting[$ID]['scheduleKey'] = $scheduleKey;
		}
		return $scheduleWaiting;
	}

	function killScheduleBackup($historyID){
		$where = array(
			'query' =>  "historyID = ':historyID'",
			'params' => array(
				':historyID' => $historyID
				)
			);
		if(!$this->isBackupWaiting($historyID)){
			return false;
		}
		DB::update("?:history", array( 'status' => 'completed'), $where);
		DB::update('?:history_additional_data', array('status' => 'error','error' => 'task_killed_another_same_schedule_waiting' ,'errorMsg' => 'This task is killed because the website is being waited for backing up in another schedule.'), $where);
	}

	function isBackupWaiting($historyID){
		$where = array(
			'query' =>  "historyID = ':historyID'",
			'params' => array(
				':historyID' => $historyID
				)
			);
		$currentStatus = DB::getField('?:history', 'status', $where);
		if ($currentStatus == 'scheduled' || $currentStatus == 'initiated') {
			return true;
		}
		return false;
	}

	function initiateScheduleBackup($historyID, $currentHistoryID = false){
		if ($currentHistoryID==false) {
			$where = array(
			'query' =>  "historyID = ':historyID'",
			'params' => array(
				':historyID' => $historyID
				)
			);
			DB::update("?:history", array( 'status' => 'pending'), $where);
		} else if ($historyID < $currentHistoryID) {
			$this->postponeCurrentScheduleBackup($currentHistoryID);
			$where = array(
			'query' =>  "historyID = ':historyID'",
					'params' => array(
					':historyID' => $historyID
					)
				);
			DB::update("?:history", array( 'status' => 'pending'), $where);
		} else {
			// $this->postponeCurrentScheduleBackup($historyID);
			$where = array(
			'query' =>  "historyID = ':historyID'",
			'params' => array(
				':historyID' => $currentHistoryID
				)
			);
			DB::update("?:history", array( 'status' => 'pending'), $where);
		}
	}
}